from .Vector import Vector

# Class represents a single point in 3d space
class Point:
    def __init__(self, x, y, z):
        self.x = x
        self.y = y
        self.z = z

    # Add two points together
    def __add__(self, other):
        return Point(self.x + other.x, self.y + other.y, self.z + other.z)

    # Subtracts two points
    def __sub__(self, other):
        return Vector(self.x - other.x, self.y - other.y, self.z - other.z)
    
    def __str__(self):
        return "Point(%s, %s, %s)" % (str(self.x), str(self.y), str(self.z))



class Color:
    def __init__(self, r, g, b, alpha=255):
        self.r = r
        self.g = g
        self.b = b
        self.alpha = alpha

        if self.r < 0 or self.g < 0 or self.b < 0 or self.alpha < 0:
            raise ValueError("color values can't be below 0")
        if self.r > 255 or self.g > 255 or self.b > 255 or self.alpha > 255:
            raise ValueError("color values can't be above 255")
    
    # Get the floating point representation between 0 and 1
    def scale_down(self):
        return Color(self.r / 255, self.g / 255, self.b / 255, self.alpha / 255)


from math import sqrt

# Represents a 3d vector
class Vector:
    def __init__(self, x, y, z):
        self.x = x
        self.y = y
        self.z = z
    
    def __add__(self, other):
        return Vector(self.x + other.x, self.y + other.y, self.z + other.z)

    def __sub__(self, other):
        return Vector(self.x - other.x, self.y - other.y, self.z - other.z)

    def __mul__(self, scalar):
        return Vector(self.x * scalar, self.y * scalar, self.z * scalar)
    
    def __len__(self):
        return sqrt(self.x * self.x + self.y * self.y + self.z * self.z)
    
    def normalize(self):
        length = self.__len__()
        self.x /= length
        self.y /= length
        self.z /= length

    def dot(self, other):
        return self.x * other.x + self.y * other.y + self.z * other.z

    def cross(self, other):
        return Vector(self.y * other.z - self.z * other.y, self.z * other.x - self.x * other.z, self.x * other.y - self.y * other.x)
    
    def __str__(self):
        return "Vector(%s, %s, %s)" % (str(self.x), str(self.y), str(self.z))
